@@ -61,6 +61,8 @@ static int setup_leave(struct setup_backend *backend)
 	/* dangerous, we can't wipe all the memory */
 	if (!memory_unsafe)
 		munlockall();
+
+	return 0;
 }
 
 /*
@@ -165,6 +167,7 @@ static void get_key(struct crypt_options *options, char *prompt, char **key, int
 		}
 		if(options->key_file)
 			close(fd);
+		pass[i] = 0;
 		*key = pass;
 		*passLen = i;
 	}
@@ -286,7 +289,7 @@ static int parse_into_name_and_mode(const char *nameAndMode, char *name,
 	int r;
 
 	if(sscanf(nameAndMode,sp1, name, mode) != 2) {
-		if((r =sscanf(nameAndMode,sp2,name)) == 1) {
+		if((r = sscanf(nameAndMode,sp2,name)) == 1) {
 			strncpy(mode,"cbc-plain",10);
 		} 
 		else {
@@ -516,6 +519,13 @@ static int __crypt_luks_open(int arg, struct setup_backend *backend, struct cryp
 	char *dmCipherSpec;
 	int r;
 	
+	if (get_device_infos(options->device, &infos) < 0) {
+		set_error("Can't get device information.\n");
+		r = -ENOTBLK; goto out;
+	}
+	if (infos.readonly)
+		options->flags |= CRYPT_FLAG_READONLY;
+
 	options->key_size = 0; // FIXME, define a clean interface some day.
 	get_key(options,"Enter LUKS passphrase: ",&password,&passwordLen);
 	if(!password) {
@@ -523,35 +533,38 @@ static int __crypt_luks_open(int arg, struct setup_backend *backend, struct cryp
 	}
 	if(LUKS_open_any_key(options->device, password, passwordLen, &hdr, &mk, backend) < 0) {
 		set_error("No key available with this passphrase.\n");
-		r = -EPERM; goto out;
+		r = -EPERM; goto out1;
 	}
 	
 	options->offset = hdr.payloadOffset;
  	asprintf(&dmCipherSpec, "%s-%s", hdr.cipherName, hdr.cipherMode);
+	if(!dmCipherSpec) {
+		r = -ENOMEM;
+		goto out2;
+	}
 	options->cipher = dmCipherSpec;
 	options->key_size = mk.keyLength;
-	options->skip = 0; options->flags = 0;
+	options->skip = 0;
 
-	if (get_device_infos(options->device, &infos) < 0) {
-		set_error("Can't get device information.\n");
-		r = -ENOTBLK; goto out;
-	}
 	options->size = infos.size;
 	if (!options->size) {
 		set_error("Not a block device.\n");
-		r = -ENOTBLK; goto out;
+		r = -ENOTBLK; goto out2;
 	}
 	if (options->size <= options->offset) {
 		set_error("Invalid offset");
-		/*		printf("s: %d, o: %d\n",options->size, options->offset); */
-		r = -EINVAL; goto out;
+		r = -EINVAL; goto out2;
 	}
 	options->size -= options->offset;
 	backend->create(0, options, mk.key);
+
 	r = 0;
-out:	
-	memset(&mk,0,sizeof(mk));
+ out2:
+	free(dmCipherSpec);
+ out1:
 	safe_free(password);	
+ out:
+	memset(&mk,0,sizeof(mk));
 	return r;
 }
 
@@ -680,7 +693,7 @@ int crypt_luksOpen(struct crypt_options *options)
 
 int crypt_luksDelKey(struct crypt_options *options)
 {
-	LUKS_del_key(options->device, options->key_slot);
+	return LUKS_del_key(options->device, options->key_slot);
 }
 
 int crypt_luksAddKey(struct crypt_options *options)
